package com.qhtx.eta.common.interceptor;

import com.qhtx.eta.common.eunms.ETARetryEnum;
import com.qhtx.eta.common.eunms.ErrorEnum;
import okhttp3.*;
import org.jetbrains.annotations.NotNull;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

public class RetryInterceptor implements Interceptor {
    private static final Logger log = LoggerFactory.getLogger(RetryInterceptor.class);
    /**
     * 重试次数
     */
    private int retryTimes;
    /**
     * 重试间隔
     */
    private int retryInterval;

    private RetryInterceptor(int retryInterval, int retryTimes) {
        this.retryInterval = retryInterval;
        this.retryTimes = retryTimes;
    }

    @NotNull
    @Override
    public Response intercept(@NotNull Chain chain) throws IOException {
        AtomicInteger retryCount = new AtomicInteger(0);
        Request request = chain.request();
        Response response = null;
        while (retryCount.getAndIncrement() < retryTimes) {
            try {
                response = chain.proceed(request);
                if (!isSuccessful(response) && shouldRetry(response.code())) {
                    TimeUnit.MILLISECONDS.sleep((long) retryInterval * retryCount.get());
                    log.warn("尝试重新请求第 " + retryCount + " 次,上次应答为:[{}:{}]", response.code(), response.message());
                    response.close();
                    continue;
                }
                return response;
            } catch (IOException var1) {
                log.warn("请求失败,应答结果为:[{}]", var1.getMessage());
            } catch (InterruptedException var2) {
                log.warn("请求等待中断异常:[{}]", var2.getMessage());
            }
        }
        if (response != null) {
            throw new IOException(String.format("请求失败,最后一次应答结果为:[%d:%s]", response.code(), response.message()));
        }
        throw new IOException(String.format("请求失败,最后一次应答结果为:[%s]", ErrorEnum.RESPONSE_EMPTY_ERROR.getMsg()));
    }


    public static class Builder {
        /**
         * 重试次数
         */
        private int retryTimes;
        /**
         * 重试间隔
         */
        private int retryInterval;


        public Builder retryTimes(int retryTimes) {
            this.retryTimes = retryTimes;
            return this;
        }

        public Builder retryInterval(int retryInterval) {
            this.retryInterval = retryInterval;
            return this;
        }

        public RetryInterceptor build() {
            return new RetryInterceptor(retryInterval, retryTimes);
        }
    }

    private boolean isSuccessful(Response response) {
        return response.isSuccessful();
    }


    private boolean shouldRetry(int code) {
        return ETARetryEnum.needRetry(code);
    }


}
