package com.qhtx.eta.domain.service.impl;

import com.google.common.collect.Lists;
import com.qhtx.eta.common.utils.MapUtils;
import com.qhtx.eta.domain.convert.EtaApiQuotaDataConverter;
import com.qhtx.eta.domain.convert.EtaApiQuotaInfoConverter;
import com.qhtx.eta.domain.entity.LinkFieldDTO;
import com.qhtx.eta.domain.entity.QuotaDataDTO;
import com.qhtx.eta.domain.entity.QuotaDataUpdateBO;
import com.qhtx.eta.domain.entity.QuotaInfoDTO;
import com.qhtx.eta.domain.enums.ETADataStatus;
import com.qhtx.eta.domain.service.ETAQuotaService;
import com.qhtx.eta.common.config.DataSourceProperties;
import com.qhtx.eta.common.constant.ETAConstants;
import com.qhtx.eta.common.exception.ETAException;
import com.qhtx.eta.infra.entity.ods.EtaApiQuotaData;
import com.qhtx.eta.infra.entity.ods.EtaApiQuotaInfo;
import com.qhtx.eta.infra.enums.DimTranFieldType;
import com.qhtx.eta.infra.service.EtaApiQuotaDataService;
import com.qhtx.eta.infra.service.EtaApiQuotaInfoService;
import com.qhtx.eta.infra.service.OdsDimService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

@Slf4j
@Service
public class ETAQuotaServiceImpl implements ETAQuotaService {
    @Resource
    private EtaApiQuotaInfoService etaApiQuotaInfoService;

    @Resource
    private TDampLinkService tDampLinkService;
    @Resource
    private EtaApiQuotaDataService etaApiQuotaDataService;
    @Resource(name = "ETAInfoThreadPool")
    private ThreadPoolExecutor etaInfoThreadPool;

    @Resource(name = "IndexSyncThreadPool")
    private ThreadPoolExecutor indexSyncThreadPool;
    @Resource
    private OdsDimService odsDimService;
    @Resource
    private DataSourceProperties dataSourceProperties;

    @Override
    public void syncQuota(List<QuotaInfoDTO> quotaInfoList) {
        List<EtaApiQuotaInfo> quotaList = EtaApiQuotaInfoConverter.INSTANCE.convertEntityToBoList(quotaInfoList);
        List<CompletableFuture<Void>> futures = new ArrayList<>();
        Lists.partition(quotaList, dataSourceProperties.getQuota().getChunkSize()).forEach(subList -> {
            CompletableFuture<Void> future = CompletableFuture.runAsync(() -> etaApiQuotaInfoService.batchInsertOrUpdate(subList), etaInfoThreadPool);
            futures.add(future);
        });
        int size = futures.size();
        CompletableFuture.allOf(futures.toArray(new CompletableFuture[size])).join();
    }

    @Override
    public void syncData(List<QuotaDataDTO> quotaDataList) {
        List<EtaApiQuotaData> dataList = EtaApiQuotaDataConverter.INSTANCE.convertEntityToBoList(quotaDataList);
        dataList.sort(Comparator.comparing(EtaApiQuotaData::getDataTime));
        List<List<EtaApiQuotaData>> chunks = Lists.partition(dataList, dataSourceProperties.getData().getChunkSize());
        for (int i = 0; i < chunks.size(); i++) {
            List<EtaApiQuotaData> subList = chunks.get(i);
            int current = current(i);
            try {
                etaApiQuotaDataService.batchInsert(subList);
                log.info("完成[{}]指标数据第{}部分数据同步，数据量:{}", subList.get(0).getUniqueCode(), current, subList.size());
            } catch (ETAException e) {
                log.error("数据库操作失败[{}]第" + current + "批次,{}", subList.get(0).getUniqueCode(), e.getErrorMessage());
                break;
            } catch (Exception e) {
                log.error("数据库操作失败[{}]第" + current + "批次", subList.get(0).getUniqueCode());
                break;
            }
        }
        log.info("================================ 同步接口指标数据完成 ================================ ");
    }

    private int current(int i) {
        return i + 1;
    }

    @Override
    public List<QuotaDataUpdateBO> getQuotaInfoListForUpdate(QuotaInfoDTO quotaInfo) {
        EtaApiQuotaInfo etaApiQuotaInfo = EtaApiQuotaInfoConverter.INSTANCE.convertEntityToBo(quotaInfo);
        //一次性获取所有的指标的最新数据，减少数据库请求开销
        List<EtaApiQuotaData> dataList = etaApiQuotaDataService.getAllLatestDataByUniqueCode();
        List<EtaApiQuotaInfo> quotaInfoList = etaApiQuotaInfoService.queryQuotaList(etaApiQuotaInfo).stream().filter(info -> info.getLatestDate() != null && info.getUniqueCode() != null).collect(Collectors.toList());
        List<List<QuotaDataUpdateBO>> allResults;
        List<CompletableFuture<List<QuotaDataUpdateBO>>> completableFutureList = Lists.partition(quotaInfoList, dataSourceProperties.getData().getChunkSize()).stream().map(subList -> CompletableFuture.supplyAsync(() -> dataUpdate(subList, dataList), etaInfoThreadPool)).collect(Collectors.toList());
        try {
            int size = completableFutureList.size();
            allResults = CompletableFuture.allOf(completableFutureList.toArray(new CompletableFuture[size])).thenApply(voidValue -> completableFutureList.stream().map(CompletableFuture::join).collect(Collectors.toList())).get();
        } catch (Exception e) {
            log.warn("{}指标数据更新失败{}{}", ETAConstants.LINE_SEPARATOR, e.getMessage(), ETAConstants.LINE_SEPARATOR);
            return Collections.emptyList();
        }
        return allResults.stream().flatMap(List::stream).collect(Collectors.toList());
    }

    @Override
    public List<QuotaInfoDTO> getQuotaList() {
        long total = etaApiQuotaInfoService.countQuota();
        LinkFieldDTO dimensionField = tDampLinkService.getLinkField(DimTranFieldType.INDEX_NAME.getValue());
        LinkFieldDTO metricField = tDampLinkService.getLinkField(DimTranFieldType.METRIC.getValue());
        String enName = StringUtils.substring(dimensionField.getFiledName(), dimensionField.getFiledName().indexOf("(") + 1, dimensionField.getFiledName().indexOf(")"));
        String cnMetric = StringUtils.substring(metricField.getFiledName(), 0, metricField.getFiledName().indexOf("("));
        if (total > ETAConstants.INDEX_SYNC_LIMIT) {
            List<CompletableFuture<List<QuotaInfoDTO>>> futures = new ArrayList<>();
            long chunk = (total + ETAConstants.INDEX_SYNC_LIMIT - 1) / ETAConstants.INDEX_SYNC_LIMIT;
            for (int i = 0; i < chunk; i++) {
                int current = i;
                CompletableFuture<List<QuotaInfoDTO>> future = CompletableFuture.supplyAsync(() -> {
                    try {
                        int offset = current * ETAConstants.INDEX_SYNC_LIMIT;
                        List<QuotaInfoDTO> list = EtaApiQuotaInfoConverter.INSTANCE.convertEntityToDTOList(etaApiQuotaInfoService.queryQuotaListPage(offset, ETAConstants.INDEX_SYNC_LIMIT));
                        list.forEach(quotaInfoDTO -> {
                            String FiledName = MapUtils.underscoreToCamelCase(enName, true);
                            try {
                                Field field = QuotaInfoDTO.class.getDeclaredField(FiledName);
                                field.setAccessible(true);
                                String value = String.valueOf(field.get(quotaInfoDTO));
                                String elementVal = StringUtils.join(value, ETAConstants.COLON, cnMetric);
                                quotaInfoDTO.setElementVal(elementVal);
                            } catch (Exception e) {
                                log.error("获取指标度量字段失败", e);
                            }
                        });
                        return list;
                    } catch (Exception e) {
                        log.error("获取指标列表失败", e);
                        return Collections.emptyList();
                    }
                }, indexSyncThreadPool);
                futures.add(future);
            }
            return futures.stream().map(CompletableFuture::join).flatMap(List::stream).collect(Collectors.toList());
        } else {
            List<QuotaInfoDTO> list = EtaApiQuotaInfoConverter.INSTANCE.convertEntityToDTOList(etaApiQuotaInfoService.getAllQuotaList());
            list.forEach(quotaInfoDTO -> {
                String FiledName = MapUtils.underscoreToCamelCase(enName, true);
                try {
                    Field field = QuotaInfoDTO.class.getDeclaredField(FiledName);
                    field.setAccessible(true);
                    String value = String.valueOf(field.get(quotaInfoDTO));
                    String elementVal = StringUtils.join(value, ETAConstants.COLON, cnMetric);
                    quotaInfoDTO.setElementVal(elementVal);
                } catch (Exception e) {
                    log.error("获取指标度量字段失败", e);
                }
            });
            return list;
        }
    }

    @Override
    public void LinkedCodeMessage(long minOperationTime) {

        odsDimService.odsDimDataUpdate(ETAConstants.ODS_DATA_TABLE_NAME, minOperationTime, 1);
    }

    private List<QuotaDataUpdateBO> dataUpdate(List<EtaApiQuotaInfo> quotaInfoList, List<EtaApiQuotaData> dataList) {
        return quotaInfoList.stream().map(info -> {
            QuotaDataUpdateBO quotaDataUpdateBO = new QuotaDataUpdateBO();
            quotaDataUpdateBO.setUniqueCode(info.getUniqueCode());
            quotaDataUpdateBO.setFrequency(info.getFrequency());
            List<EtaApiQuotaData> etaApiQuotaDataList = dataList.stream().filter(data -> data.getUniqueCode().equals(info.getUniqueCode())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(etaApiQuotaDataList)) {
                quotaDataUpdateBO.setStatus(ETADataStatus.NEW);
                return quotaDataUpdateBO;
            }
            quotaDataUpdateBO.setCurrentDate(new SimpleDateFormat(ETAConstants.DATE_PATTERN).format(etaApiQuotaDataList.get(0).getDataTime()));
            if (compareDate(etaApiQuotaDataList.get(0).getDataTime(), info.getLatestDate())) {
                quotaDataUpdateBO.setStatus(ETADataStatus.UPDATE);
                return quotaDataUpdateBO;
            }
            quotaDataUpdateBO.setStatus(ETADataStatus.LATEST);
            return quotaDataUpdateBO;
        }).collect(Collectors.toList());
    }

    /**
     * 比较时间 src 小于 des 返回 true 否则返回false
     *
     * @param src
     * @param des
     * @return boolean
     */
    private boolean compareDate(Date src, Date des) {
        LocalDate co1 = src.toInstant().atZone(ZoneId.of(ETAConstants.TIME_ZONE)).toLocalDate();
        LocalDate co2 = des.toInstant().atZone(ZoneId.of(ETAConstants.TIME_ZONE)).toLocalDate();
        return co1.isBefore(co2);
    }

}
